#include "param.h"

float4   cnt    : register(c0);
sampler  t0     : register(s0);


float3 rotate_pos(float3 d, float rot) {
  float3x3 mtx = {
    cos(rot),  sin(rot), 0,
    sin(rot), -cos(rot), 0,
    0,       0,      1
  };
  d = mul(mtx, d).zxy;
  return mul(mtx, d).xzy;
}

float torus(float3 p) {
  float3 d = ( (1024 + p) % 1) - 0.5;
  float rot = (length(p + cnt.x));
  d = rotate_pos(d, rot);
  float  R  = 0.4;
  float  rr = 0.05;
  float g = length(d.xy) - R * R;
  g = (g * g) + (d.z * d.z);
  return sqrt(g) - rr;
}

float iso(float3 p) {
  return torus(p);
}


//raycast
float raycast(float3 eye, float3 dir) {
  float t = 0;
  for(int i = 0; i < 100; i++) {
    float3 temp = eye + dir * t;
    float g = iso(temp);
    t += g * 0.8;
  }
  return t;
}

//get normal
float3 getnormal(float3 ip) {
  float  iit    = iso(ip);
  float3 h      = float3(0.01, 0, 0);
  return normalize(float3(
    iit - iso(ip + h.xyy),
    iit - iso(ip + h.yxy),
    iit - iso(ip + h.yyx)));
}

float4 ps_main(float2 vp : vpos) : color {
  float3 light  = normalize(float3(-0.7,  0.5,  0.7));
  float2 uv = -1 + 2 * vp / (float2(ScreenX, ScreenY));
  if(abs(uv.y) > 0.8) return 0;
  float time  = cnt.x;
  float rotate = (time + sin(time * 0.5)) * 0.1;
  float3x3 mtx = {
    cos(rotate),  sin(rotate), 0,
    sin(rotate), -cos(rotate), 0,
    0,            0,           1
  };
  
  //rotate ray direction
  float3 dir    = normalize(
    float3(uv.x * 1.25 - sin(time * 3) * 0.01,
           uv.y        + cos(time * 2) * 0.01,
      1 + sin(4 * sin(time * .3)) * .1));
  if( (time % 20) > 10) dir = mul(mtx, dir).yzx;
  dir = mul(mtx, dir).yzx;
  dir = mul(mtx, dir);
  
  //cast
  float3 eye    = float3(0, 0, cnt.x);
  float  t      = raycast(eye, dir);
  float3 normal = dot(light, getnormal(eye + dir * t));
  float3 rgb    = max(float3(0.9, 0.4, 0.1) * normal, 0) + dir * 0.3;
  float4 col    = 0;
  col    = float4(rgb + (t * 0.06), 1);
  col   -= dot(uv, uv) * 0.5;
  return float4(col.xyz, t) * min((time % 10) * 0.75, 1);
}


float4 ps_main_f(float2 vp : vpos, float2 uv : texcoord0) : color {
  float4 argb = tex2D(t0, uv);
  return argb;
}


